package com.jiuqi.util;

import java.util.ArrayList;
import java.util.List;

/* loaded from: com_jiuqi_util.jar:com/jiuqi/util/MemStream.class */
public class MemStream extends Stream {
    private static final int PAGE_SIZE = 1024;
    private List pages = new ArrayList();
    private long size = 0;
    private int curPageNo = 0;
    private int curPos = 0;

    private byte[] curPage() {
        if (this.curPageNo < this.pages.size()) {
            return (byte[]) this.pages.get(this.curPageNo);
        }
        return null;
    }

    @Override // com.jiuqi.util.Stream
    public void setSize(long newSize) throws StreamException {
        if (newSize < 0) {
            throw new StreamException(StreamException.STREAM_SIZEERROR);
        }
        this.size = newSize;
        int pagecount = (int) (this.size / 1024);
        if (this.size % 1024 > 0) {
            pagecount++;
        }
        if (pagecount > this.pages.size()) {
            int k = pagecount - this.pages.size();
            for (int i = 0; i < k; i++) {
                this.pages.add(new byte[PAGE_SIZE]);
            }
        } else {
            int k2 = this.pages.size() - pagecount;
            for (int i2 = 0; i2 < k2; i2++) {
                this.pages.remove(this.pages.size() - 1);
            }
        }
        if ((this.curPageNo * PAGE_SIZE) + this.curPos > this.size) {
            this.curPageNo = (int) (this.size / 1024);
            this.curPos = (int) (this.size % 1024);
        }
    }

    @Override // com.jiuqi.util.Stream
    public int read(byte[] buffer, int pos, int count) throws StreamException {
        if (pos < 0 || count < 0) {
            throw new StreamException(StreamException.STREAM_READERROR);
        }
        if (count == 0 || pos >= buffer.length) {
            return 0;
        }
        if (pos + count >= buffer.length) {
            count = buffer.length - pos;
        }
        if ((this.size - (this.curPageNo * PAGE_SIZE)) - this.curPos < count) {
            count = (int) ((this.size - (this.curPageNo * PAGE_SIZE)) - this.curPos);
        }
        int readed = 0;
        while (readed < count) {
            byte[] page = curPage();
            int canread = count - readed;
            if (canread > PAGE_SIZE - this.curPos) {
                canread = PAGE_SIZE - this.curPos;
            }
            System.arraycopy(page, this.curPos, buffer, pos, canread);
            pos += canread;
            readed += canread;
            this.curPos += canread;
            if (this.curPos == PAGE_SIZE) {
                this.curPageNo++;
                this.curPos = 0;
            }
        }
        return count;
    }

    @Override // com.jiuqi.util.Stream
    public byte read() throws StreamException {
        byte[] page = curPage();
        if (page != null) {
            byte rt = page[this.curPos];
            this.curPos++;
            if (this.curPos == PAGE_SIZE) {
                this.curPos = 0;
                this.curPageNo++;
            }
            return rt;
        }
        throw new StreamException(StreamException.STREAM_READERROR);
    }

    @Override // com.jiuqi.util.Stream
    public int write(byte[] buffer, int pos, int count) throws StreamException {
        if (pos < 0 || count < 0) {
            throw new StreamException(StreamException.STREAM_READERROR);
        }
        if (count == 0 || buffer == null || pos >= buffer.length) {
            return 0;
        }
        if (pos + count >= buffer.length) {
            count = buffer.length - pos;
        }
        long p = getPosition();
        int writed = 0;
        while (writed < count) {
            byte[] page = curPage();
            if (page == null) {
                page = new byte[PAGE_SIZE];
                this.pages.add(page);
            }
            int canwrite = PAGE_SIZE - this.curPos;
            if (canwrite > count - writed) {
                canwrite = count - writed;
            }
            System.arraycopy(buffer, pos, page, this.curPos, canwrite);
            writed += canwrite;
            p += canwrite;
            if (p > this.size) {
                this.size = p;
            }
            this.curPos += canwrite;
            pos += canwrite;
            if (this.curPos == PAGE_SIZE) {
                this.curPos = 0;
                this.curPageNo++;
            }
        }
        return count;
    }

    @Override // com.jiuqi.util.Stream
    public void write(byte value) throws StreamException {
        byte[] page = curPage();
        if (page == null) {
            page = new byte[PAGE_SIZE];
            this.pages.add(page);
        }
        page[this.curPos] = value;
        this.curPos++;
        if (this.curPos >= PAGE_SIZE) {
            this.curPos = 0;
            this.curPageNo++;
        }
        if ((this.curPageNo * PAGE_SIZE) + this.curPos > this.size) {
            this.size++;
        }
    }

    @Override // com.jiuqi.util.Stream
    public long seek(long offset, int origin) throws StreamException {
        switch (origin) {
            case 0:
                if (offset < 0 || offset > this.size) {
                    throw new StreamException(StreamException.STREAM_SEEKERROR);
                }
                this.curPageNo = (int) (offset / 1024);
                this.curPos = (int) (offset % 1024);
                break;
            case 1:
                long x = offset + (this.curPageNo * PAGE_SIZE) + this.curPos;
                if (x < 0 || x > this.size) {
                    throw new StreamException(StreamException.STREAM_SEEKERROR);
                }
                this.curPageNo = (int) (x / 1024);
                this.curPos = (int) (x % 1024);
                break;
                break;
            case 2:
                if (offset > 0 || offset + this.size < 0) {
                    throw new StreamException(StreamException.STREAM_SEEKERROR);
                }
                long x2 = offset + this.size;
                this.curPageNo = (int) (x2 / 1024);
                this.curPos = (int) (x2 % 1024);
                break;
                break;
        }
        return (this.curPageNo * PAGE_SIZE) + this.curPos;
    }
}
